package edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Embeddable;
import javax.persistence.GeneratedValue;
import javax.persistence.Transient;

@Embeddable
public class Report {

	private int reportID;
	private Map<TestResults, RawImage> correspondingTestImage;
	private DatabaseAdministrator importedBy;
	private Date importedOn;
	private RawImage independentImage;
	private TestResults independentTest;
	private List<String> processedImagePath;
	private List<ProcessType> processesDone;
	private List<Exception> processingErrors;
	
	/**
	 * Constructor for a report
	 */
	public Report() {
		correspondingTestImage = new HashMap<TestResults, RawImage>();
	}

	@GeneratedValue
	public int getReportID() {
		return reportID;
	}

	public void setReportID(int reportID) {
		this.reportID = reportID;
	}

	/**
	 * Returns the raw image associated with all test results
	 * @return A map with test results as keys and raw images as values
	 */
	@Transient
	public Map<TestResults, RawImage> getCorrespondingTestImage() {
		return correspondingTestImage;
	}
	/**
	 * Returns the database administrator responsible for creating the report
	 * @return DatabaseAdministrator associated with the report
	 */
	@Transient
	public DatabaseAdministrator getImportedBy() {
		return importedBy;
	}
	/**
	 * Returns the data on which the report was generated
	 * @return Date on which the report was generated
	 */
	public Date getImportedOn() {
		return importedOn;
	}
	/**
	 * Returns a raw image used in the report
	 * @return RawImage for the raw image used
	 */
	@Transient
	public RawImage getIndependentImage() {
		return independentImage;
	}
	/**
	 * Getter method for retrieving the independent test result
	 * @return TestResults for a single test
	 */
	@Transient
	public TestResults getIndependentTest() {
		return independentTest;
	}
	/**
	 * Getter method for retrieving the list of processing types on the image
	 * @return List of Processing types on image.
	 */
	@Transient
	public List<ProcessType> getProcessesDone() {
		return processesDone;
	}

	/**
	 * Getter method for the saved image path for the processed images
	 * @return List of Strings of the file path locations for the processed images
	 */
	@Transient
	public List<String> getProcessedImagePath() {
		return processedImagePath;
	}

	/**
	 * Getter method for the errors that occurred during image processing steps
	 * @return List of Exceptions received during image processing
	 */
	@Transient
	public List<Exception> getProcessingErrors() {
		return processingErrors;
	}


}
