package edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.test;

import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.Data;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.DefaultProcessing;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.Demographic;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.ProcessType;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.ProcessingException;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.RawImage;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.Report;
import edu.jhu.cs.oose.fall2014.group16.brainAnalysis.model.DatabaseAdministrator;

public class DatabaseAdministratorTest {
	
	DatabaseAdministrator admin;
	
	@Before
	public void setUp() throws Exception {
		admin = new DatabaseAdministrator("admin", "password");
	}
	
	/**
	 * Try to insert null data into the database. The system should return a
	 * report that informs the user of the error
	 */
	@Test
	public void importData() {
		File file = new File("RealData.txt");
		Data data = new Demographic(file);
		ArrayList<Data> list = new ArrayList<Data>();
		list.add(data);
		Report report = admin.importData(list);
		List<Exception> errors = report.getProcessingErrors();
		assertEquals(errors.size(), 0);
		assertEquals(report.getImportedBy(), this);
	}
	
	/**
	 * Try to insert null data into the database. The system should return a
	 * report that informs the user of the error
	 */
	@Test
	public void importDataNull() {
		Report errorReport = admin.importData(null);
		List<Exception> errors = errorReport.getProcessingErrors();
		assertEquals(errors.size(), 1);
		assertTrue(errors.get(0) instanceof ProcessingException);
		assertEquals(errors.get(0).getMessage(), "No data selected");
	}
	
	/**
	 * Try to insert empty list into the database. The system should return a
	 * report that informs the user of the error
	 */
	@Test
	public void importEmptyDataList() {
		Report errorReport = admin.importData(new ArrayList<Data>());
		List<Exception> errors = errorReport.getProcessingErrors();
		assertEquals(errors.size(), 1);
		assertTrue(errors.get(0) instanceof ProcessingException);
		assertEquals(errors.get(0).getMessage(), "No data selected");
	}
	
	/**
	 * Try to insert a list that contains bad (null) data into the database. The system should return a
	 * report that informs the user of the error
	 */
	@Test
	public void importEmptyData() {
		Data data = new RawImage(null);
		ArrayList<Data> list = new ArrayList<Data>();
		list.add(data);
		Report errorReport = admin.importData(list);
		List<Exception> errors = errorReport.getProcessingErrors();
		assertEquals(errors.size(), 1);
		assertTrue(errors.get(0) instanceof ProcessingException);
		assertEquals(errors.get(0).getMessage(), "No file was seleced or the file is empty");
	}
	
	/**
	 * Try to insert a list that contains bad (empty file) data into the database. The system should return a
	 * report that informs the user of the error
	 */
	@Test
	public void importEmptyDataFile() {
		File file = new File("EmptyData.txt");
		Data data = new Demographic(file);
		ArrayList<Data> list = new ArrayList<Data>();
		list.add(data);
		Report errorReport = admin.importData(list);
		List<Exception> errors = errorReport.getProcessingErrors();
		assertEquals(errors.size(), 1);
		assertTrue(errors.get(0) instanceof ProcessingException);
		assertEquals(errors.get(0).getMessage(), "No file was seleced or the file is empty");
	}
	
	/**
	 * Try to change default processing to a list of one process type.  
	 */
	@Test
	public void testDefaultProcessing() {
		DefaultProcessing defaultProcess = new DefaultProcessing();
		assertEquals(defaultProcess.getCurrentlySelectedProcesses().size(), 0);
		ProcessType data = new ProcessType("name", "path");
		ArrayList<ProcessType> list = new ArrayList<ProcessType>();
		list.add(data);
		admin.changeDefaultProcessing(list);
		assertEquals(defaultProcess.getCurrentlySelectedProcesses().size(), 1);
		assertTrue(defaultProcess.getCurrentlySelectedProcesses().contains(data));
	}
	
	/**
	 * Try to change default processing to an empty list of process types.  
	 */
	@Test
	public void testEmptyDefaultProcessing() {
		DefaultProcessing defaultProcess = new DefaultProcessing();
		assertEquals(defaultProcess.getCurrentlySelectedProcesses().size(), 0);
		ArrayList<ProcessType> list = new ArrayList<ProcessType>();
		admin.changeDefaultProcessing(list);
		assertEquals(defaultProcess.getCurrentlySelectedProcesses().size(), 0);
	}
	
	/**
	 * Try to change default processing to null.  Default Processing should be empty.
	 */
	@Test
	public void testNullDefaultProcessing() {
		DefaultProcessing defaultProcess = new DefaultProcessing();
		admin.changeDefaultProcessing(null);
		assertEquals(defaultProcess.getCurrentlySelectedProcesses().size(), 0);
	}

}
